#!/system/bin/sh
##################################################
# xt_HL module shell v1.0                        #
# indi (c) 2o16 <programming4food<dog>gmail.com> #
##################################################

TTL=64
NET_NAME=rmnet+



SCRIPT=`realpath $0`
SCRIPTPATH=`dirname $SCRIPT`
MODULEPATH=$SCRIPTPATH/modules
KERNEL=`uname -r`
MODULE=$MODULEPATH/$KERNEL-xt_HL.ko



echo "\
 ------------------------------------------------
  xt_HL module shell v1.0
  indi (c) 2o16 <programming4food<dog>gmail.com>
 ------------------------------------------------
 You using kernel: $KERNEL
"

check_iptables_ttl() {
  echo -n " [.] Checking kernel features: "
  cat /proc/net/ip_tables_targets | grep TTL > /dev/null
  CHK_RESULT=$?
  [ $CHK_RESULT -eq 0 ] && echo "OK" || echo "NONE"
  return $CHK_RESULT
}

check_kernel_module() {
  echo -n " [.] Checking kernel module:   "
  lsmod | grep xt_HL  > /dev/null
  CHK_RESULT=$?
  [ $CHK_RESULT -eq 0 ] && echo "OK" || echo "NONE"
  return $CHK_RESULT
}

do_iptables_rule() {
  CMD="iptables -t mangle -$1 POSTROUTING -o $NET_NAME -j TTL --ttl-set $TTL"
  if [ "$1" == "C" ]; then
    $CMD 2> /dev/null
  else
    $CMD
  fi
  CHK_RESULT=$?
  return $CHK_RESULT
}

check_iptables_rule() {
  echo -n " [.] Checking iptables rules:  "
  #iptables -S -t mangle | grep TTL > /dev/null
  do_iptables_rule C
  CHK_RESULT=$?
  [ $CHK_RESULT -eq 0 ] && echo "OK" || echo "NONE"
  return $CHK_RESULT
}

check_local_module() {
  if [ -f "$MODULE" ]; then
    echo " [i] Found module for your kernel."
  else
    echo " [!] Module for your kernel not nound."
  fi
  CHK_RESULT=$?
  return $CHK_RESULT
}

check_module() {
  MOD=`basename $1`

  check_kernel_module > /dev/null
  if [ $? -eq 0 ]; then
    rmmod xt_HL
    [ $? -ne 0 ] && echo " [*] Error unloading module."
  fi

  echo -n " [.] Checking module: $MOD ... "
  insmod $1 2> /dev/null
  CHK_RESULT=$?
  if [ $CHK_RESULT -eq 0 ]; then
    echo "GOOD"
    if [ "$MOD" != "$KERNEL-xt_HL.ko" ]; then
      echo "  ^--Rename it to \"$KERNEL-xt_HL.ko\" for use"
    fi
  else
    echo "BAD"
  fi

  check_kernel_module > /dev/null
  if [ $? -eq 0 ]; then
    rmmod xt_HL
    [ $? -ne 0 ] && echo " [*] Error unloading module."
  fi
}

ARG=`echo $1 | tr '[:upper:]' '[:lower:]'`
case "$ARG" in
  check|test)
    check_local_module
    check_iptables_ttl
    check_kernel_module
    check_iptables_rule
    ;;

  try)
    for file in $MODULEPATH/*.ko
    do
      check_module $file
    done
    ;;

  install)
    check_local_module
    if [ $? -eq 0 ]; then
      echo " [i] Installing xt_HL kernel module..."
      insmod $MODULE
    else
      check_iptables_ttl
      [ $? -ne 0 ] && echo " [*] Probably the script will not work"
    fi

    check_iptables_rule
    if [ $? -ne 0 ]; then
      echo " [.] Installing iptables ttl-set rule..."
      do_iptables_rule A
      #iptables -t mangle -A POSTROUTING -o $NET_NAME -j TTL --ttl-set $TTL
    else
      echo " [i] iptables ttl-set rule already present."
    fi
    ;;

  remove)
    check_iptables_rule
    if [ $? -eq 0 ]; then
      echo " [.] Removing iptables ttl-set rule..."
      do_iptables_rule D
    else
      echo " [i] iptables ttl-set rule are not exist."
    fi
    check_kernel_module
    if [ $? -eq 0 ]; then
      echo " [.] Removing xt_HL kernel module..."
      rmmod xt_HL
    else
      echo " [i] Kernel module not loaded."
    fi
    ;;

  *)
    echo "\
 Usage: init-xt_HL.sh [command]

  COMMANDS:
   install - install module and iptable rules
   remove  - remove module and iptable rules
   check   - check kernel for ttl-set support
   try     - try to load all available modules"
    ;;
esac

echo "End"
exit 0
